/*
 * nssudeaudio - USB Bus Class Filter Driver for fixing UDE compatibility with USBAUDIO.SYS
 *
 * Copyright (c) 2022-2023, Nefarius Software Solutions e.U.
 *  All rights reserved.
 *
 *  Redistribution and use in bytecode or binary forms, without modification,
 *  are permitted provided that the following conditions are met:
 *
 *    * Redistributions in bytecode or binary form must reproduce the above
 *      copyright notice, this list of conditions and the following disclaimer
 *      in the documentation and/or other materials provided with the
 *      distribution. This includes repository READMEs for open-source projects
 *      interfacing with this product, websites or in-application credits.
 *
 *    * Neither the name of the nssudeaudio.sys nor the names of its contributors may
 *      be used to endorse or promote products derived from or distributed
 *      with this software without specific prior written permission.
 *
 *    * Bundling or otherwise redistributing copies of binaries signed by
 *      Nefarius Software Solutions e.U. is prohibited without specific prior
 *      written permission.
 *
 *    * Personal use for research, experiment, and testing for
 *      the benefit of public knowledge, personal study, private
 *      entertainment, hobby projects, amateur pursuits, or religious
 *      observance, without any anticipated commercial application,
 *      is use for a permitted purpose.
 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 *  AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 *  ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 *  LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 *  CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 *  SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 *  INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 *  ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 *  POSSIBILITY OF SUCH DAMAGE.
 */


#include "Driver.h"
#include "Driver.tmh"


#pragma code_seg("INIT")
NTSTATUS
DriverEntry(
	_In_ PDRIVER_OBJECT  DriverObject,
	_In_ PUNICODE_STRING RegistryPath
)
{
	WPP_INIT_TRACING(DriverObject, RegistryPath);

	EventRegisterNefarius_USB_Bus_Class_Filter_Driver_for_fixing_UDE_compatibility_with_USBAUDIO_SYS();

	FuncEntry(TRACE_DRIVER);

	WDFDRIVER driver = NULL;
	WDF_OBJECT_ATTRIBUTES attributes;
	WDF_OBJECT_ATTRIBUTES_INIT(&attributes);
	attributes.EvtCleanupCallback = EvtDriverContextCleanup;

	WDF_DRIVER_CONFIG config;
	WDF_DRIVER_CONFIG_INIT(&config,
		DMF_BusFilter_DeviceAdd // provided by module
	);

	NTSTATUS status = WdfDriverCreate(
		DriverObject,
		RegistryPath,
		&attributes,
		&config,
		&driver
	);

	if (!NT_SUCCESS(status))
	{
		TraceError(
			TRACE_DRIVER,
			"WdfDriverCreate failed %!STATUS!",
			status
		);
		EventWriteFailedWithNTStatus(NULL, __FUNCTION__, L"WdfDriverCreate", status);
		goto errorExit;
	}

	DMF_BusFilter_CONFIG filterConfig;

	DMF_BusFilter_CONFIG_INIT(&filterConfig, DriverObject);

	filterConfig.DeviceType = FILE_DEVICE_BUS_EXTENDER;
	
	filterConfig.EvtDeviceAdd = EvtChildDeviceAdded;
	filterConfig.EvtDeviceQueryInterface = EvtChildDeviceQueryInterface;

	status = DMF_BusFilter_Initialize(&filterConfig);

	if (!NT_SUCCESS(status))
	{
		TraceEvents(
			TRACE_LEVEL_ERROR,
			TRACE_DRIVER,
			"DMF_BusFilter_Initialize failed %!STATUS!",
			status
		);
		EventWriteFailedWithNTStatus(NULL, __FUNCTION__, L"DMF_BusFilter_Initialize", status);
		goto errorExit;
	}

	EventWriteStartEvent(NULL, DriverObject, status);

	FuncExit(TRACE_DRIVER, "status=%!STATUS!", status);

	return status;

errorExit:
	EventUnregisterNefarius_USB_Bus_Class_Filter_Driver_for_fixing_UDE_compatibility_with_USBAUDIO_SYS();
	WPP_CLEANUP(DriverObject);
	return status;
}
#pragma code_seg()

//
// Called on driver object disposal
// 
#pragma code_seg("PAGED")
VOID
EvtDriverContextCleanup(
	_In_ WDFOBJECT DriverObject
)

{
	PAGED_CODE();

	FuncEntry(TRACE_DRIVER);

	EventWriteUnloadEvent(NULL, DriverObject);

	EventUnregisterNefarius_USB_Bus_Class_Filter_Driver_for_fixing_UDE_compatibility_with_USBAUDIO_SYS();

	//
	// Stop WPP Tracing
	//
	WPP_CLEANUP(WdfDriverWdmGetDriverObject((WDFDRIVER)DriverObject));
}
#pragma code_seg()
